# #
# $Date: 1995/10/23 08:12:37 $  $Author: frankp $   $Revision: 1.7.2.3 $ #
# #
# frankp, 2/03/94 #

#++
  gausjord.mu

	linalg::gaussJordan  --  perform the Gauss-Jordan elimination

	gaussJordan(A [,All])

	gaussJordan(A) returns A reduced to the reduced-row-echolon form.
	With option 'All', the rank, determinant (or FAIL) and a set
	of the characteristic column indices of A are returned additionally
	in form of a list.
++#

linalg::gaussJordan := proc(A)
    name linalg::gaussJordan;
    option remember;
    local R, T, t, i, j, k, d, ns, de, s, Rmult, Rdivex,
	  Rplus, Rnegate, Rzero;
begin
    if testargs() then
	if args(0) < 1 or args(0) > 2 then
	    error("wrong no of args")
	end_if;
	if A::hasProp( MatrixCat ) <> TRUE then
	    error("expecting a matrix")
	end_if;
	if not (A::coeffRing)::hasProp( IntegralDomain ) then
	    error("expecting a matrix over an integral domain")
	end_if;
	if args(0) = 2 then
	    if args(2) <> hold(All) then
	        error("expecting 'All' as option")
	    end_if
	end_if
    end_if;
  
    R:= A::coeffRing;
    T:= A::gaussElim(A);
    A:= op(T,1);

    Rdivex:= R::divex;
    Rplus:= R::_plus;
    Rnegate:= R::negate;
    Rmult:= R::_mult;
    Rzero:= R::zero;

    s:= R::iszero;
    ns:= {};
    t:= [];
    d:= A::dimen(A);
    j:= 1;
    for i from 1 to d[2] do
	if j > d[1] then break end_if;
	if s(A[j,i]) then ns:= ns union {i}
	else t:= append(t,[j,i]); j:= j + 1;
	end_if
    end_for;
    ns:= ns union {k $ k=i..d[2]}; # case r < c #
    de:= []; # sort descending #
    for i from nops(t) downto 1 do
	de:= append( de,t[i])
    end_for;

    if R::hasProp(Field) then
	t:= R::one;
	for k in de do
	    d:= A[k[1],k[2]];
	    R:= select( ns,fun(bool(args(1)>k[1])) );

	    userinfo(3,"divide row ",k[1]," by ",d);
	    for j in R do 
		A[k[1],j]:= Rdivex(A[k[1],j],d)
	    end_for;
	    A[k[1],k[2]]:= t; # = 1 #

	    for i from k[1]-1 downto 1 do
		userinfo(3,"reduce at row ",i);
		d:= Rnegate( A[i,k[2]] );
		for j in R do
		    userinfo(3,"  and column ",j);
		    A[i,j]:= Rplus(
			A[i,j], Rmult( d,A[k[1],j] )
		    )
	        end_for;
		A[i,k[2]]:= Rzero
	    end_for
	end_for
    else
        for k in de do
	    R:= select( ns,fun(bool(args(1)>k[1])) );
            d:= A[k[1],k[2]];
            de[1]:= NIL;
            d:= Rnegate(d);
            for i from k[1]-1 downto 1 do
                userinfo(3,"reduce at row ",i);
                for j in R do
                    userinfo(3,"  and column ",j);
                    A[i,j]:= Rplus(
                        Rmult(A[i,j],d), Rmult( A[i,k[2]],A[k[1],j] )
                    )
                end_for;
                for j in de do A[i,j[2]]:= Rmult(d,A[i,j[2]]) end_for;
                A[i,k[2]]:= Rzero
            end_for
        end_for
    end_if;

    if args(0) = 2 then [A,T[2],T[3],ns] else A end_if
end_proc:

# end of file #
