# #
# $Date: 1995/10/23 08:12:38 $  $Author: frankp $   $Revision: 1.38.2.10 $ #
# #
# frankp, 24/02/94 #

#++
  linsolve.mu

      linalg::linearSolve  --  solve a linear system of equations

      + linearSolve( A [,B] )

        A,B - matrices

        linearSolve(A,B) solves the systems A*x=col(B,i), i=1..ncols(B) 
        and returns a matrix representing the special solutions of the 
        ncols(B) systems.
        If B is missing the the system A[1..r,1..c-1]*c=col(A,c)
        will be solved.

      + linearSolve( A {,v,l} )

        A - matrix
        v - vector
        l - set or list with exactly ncols(A) elements

        linearSolve(A,v,l) solves the system A*x=B. If the system
        is undetermined, then the general solution of the system 
        will be expressed by the sum of a special solution and a
        linear combination of the general solution of the corresponding
        homogeneus system with scalars in l. Only those scalars are
        used which correspond to a kernel vector of A.
        If v is missing then the system A[1..r,1..c-1]*c=col(A,c)
        will be solved.

      + linearSolve( A {,v,Special,Unique} )

        A              - matrix
        v              - vector
        Special,Unique - identifier (options)

        linearSolve(A,v,Special) returns a special solution of the system.
        linearSolve(A,v,Unique) returns an unique solution if one exits,
        otherwise [] will be returned.
        If v is missing then the system A[1..r,1..c-1]*c=col(A,c)
        will be solved.


      linearSolve returns [] if the system has no solution and
      FAIL if the solution could not be computed over the coefficient
      domain of A.
      Solutions are always elements of the domain Matrix(A::coeffRing).
++#

linalg::linearSolve := proc(A,B)
    name linalg::linearSolve;
    local Flag, r, c, t, d, R, rank, i, j, k, charIndices,
	  bildIndices, ns, cB, s, RisField, MnewThis, Rdivex,
	  Rone, Rnegate, Riszero, Rmult, Rconvert;
begin
    if A::hasProp( MatrixCat ) <> TRUE then
        error("expecting a matrix")
    end_if;
    R := A::coeffRing;
    d:= A::dimen(A);
    r:= d[1]; c:= d[2];

    if A::hasProp( SquareMatrixCat ) then
	t:= A::getSuperDomain(A)
    else t:= type(A) end_if;
    MnewThis:= t::newThis;

    if testargs() and args(0) > 1 then
	if args(0) > 3 then
	    error("wrong no of args")
	end_if;
	if not contains( {hold(Unique),hold(Special)},B )
        and not contains( {DOM_SET,DOM_LIST},domtype(B) ) then
	    if not testtype( B,t ) then
                error("types of matrices don't match")
	    end_if;
	    B:= t::convert( B );
	    d:= B::dimen(B);
	    if d[1] <> r then error("invalid operands") end_if;
	    if args(0) = 3 then
		Flag:= args(3);
		if not contains( {hold(Unique),hold(Special)},Flag ) then
	            if contains( {DOM_SET,DOM_LIST},domtype(Flag) ) then
                        if nops(Flag) <> c then
                            error("list or set must have ".expr2text(c)." entries")
			end_if;
			if has(map(Flag,testtype,R),FALSE) then
                	    error("list or set contains an invalid entry")
            		end_if
		    else
			error("invalid 3rd argument")
		    end_if
                end_if
	    end_if
	elif args(0) > 2 then error("wrong no of args")
	elif contains({DOM_SET,DOM_LIST},domtype(B)) then
            if nops(B) <> c-1 then
                error("list or set must have ".expr2text(c-1)." entries")
            end_if;
	    if has(map(B,testtype,R),FALSE) then
		error("list or set contains an invalid entry")
	    end_if
	end_if
    end_if;

    if args(0) > 1 then
	if not contains( {hold(Unique),hold(Special)},B )
        and not contains( {DOM_SET,DOM_LIST},domtype(B) ) then
	    B:= t::convert(B);
	    cB:= B::dimen(B)[2];
	    A:= A::concatMatrix( A,B );
	    if args(0) = 3 then Flag:= args(3) end_if
	else
	    Flag:= B;
	    c:= c - 1; cB:= 1
	end_if
    else
	c:= c - 1; cB:= 1
    end_if;

    userinfo(1,"compute an upper triangular matrix");
    s:= A::gaussElim( A );
    rank:= s[2];

    Riszero:= R::iszero;
    userinfo(3,"test whether rank(A,b) = rank(A)");
    t:= s[1];
    if rank > 0 then
	for i from rank to c do
	    if not Riszero( t[rank,i] ) then break end_if
	end_for;
	if i > c then
	    userinfo(2,"system is overdetermined");
	    return( [] )
	end_if
    end_if;

    userinfo(1,"compute Gauss-Jordan form");
    s:= linalg::gaussJordan( A,hold(All) );
    A:= s[1];
    charIndices:= s[4] minus {j+c $ j=1..cB};
    bildIndices:= {j $ j=1..c} minus charIndices union {1};

    RisField:= R::hasProp( Field );
    if nops( charIndices ) > 0 then
	userinfo(2,"system is underdetermined");
	if Flag = hold(Unique) or cB > 1 then return( [] )
	elif RisField then
	    s:= MnewThis(c,1);
            i:= 1;
            for j from 1 to c do
                if contains( bildIndices,j ) then
                    s[j,1]:= A[i,c+1];
                    i:= i + 1
                end_if
            end_for;
	    if Flag = hold(Special) then return( s ) end_if
	elif Flag = hold(Special) then
	    Rdivex:= R::divex;
            s:= MnewThis(c,1);
            i:= 1;
            for j from 1 to c do
                if contains( bildIndices,j ) then
                    if (s[j,1]:= Rdivex( A[i,c+1],A[i,j] )) = FAIL then
                        userinfo(2,"no solution over its coefficient domain");
                        return( FAIL )
                    else i:= i + 1
                    end_if;
                end_if
            end_for
	else
	    error("sorry, can determine nullspace only over fields")
	end_if
    elif RisField then
	# the right side of A is the solution #
	return( A[1..r,c+1..c+cB] )
    else
	t:= R::divex;
	for i from 1 to r do
	    for j from c+1 to c+cB do
		if (A[i,j]:= t( A[i,j],A[i,i] )) = FAIL then
		    userinfo(2,"no solution over its coefficient domain");
                    return( FAIL )
		end_if
	    end_for
	end_for;
	return( A[1..r,c+1..c+cB] )
    end_if;

    userinfo(2,"solve the homogenous system");
    Rone:= R::one;
    Rnegate:= R::negate;
    if not contains( {DOM_LIST,DOM_SET},domtype(Flag) ) then
	j:= 0;
	for i in bildIndices do
            j:= j + 1;
            if j = 1 then
                for k in charIndices do
		    # the following order of statements is importent, e.g.
                      linsolve({x=x},{x}) -> bildIndizes=charIndizes={1},
                      hence i = k = 1, ns[1][1]:= 1 !
                    #
                    ns[k]:= [R::zero $ c ];
                    ns[k][i]:= Rnegate( A[j,k] );
		    ns[k][k]:= Rone;
                end_for
            else
                for k in charIndices do ns[k][i]:= Rnegate( A[j,k] ) end_for
            end_if
        end_for;
        t:= [];
	for k in charIndices do
	    t:= t . [MnewThis(c,1,ns[k])]
	end_for;
	[s,t]
    else
	Rmult:= R::_mult;
 	Rconvert:= R::convert;	
	j:= 0;
        for i in bildIndices do
            j:= j + 1;
            if j = 1 then
                for k in charIndices do
		    # the following order of statements is importent, e.g.
                      linsolve({x=x},{x}) -> bildIndizes=charIndizes={1},
                      hence i = k = 1, ns[1][1]:= 1 !
                    #
                    ns[k]:= [R::zero $ c ];
                    ns[k][i]:= Rmult(Rnegate( A[j,k] ),Rconvert(op(Flag,k)));
		    ns[k][k]:= Rconvert(op(Flag,k))
                end_for
            else
                for k in charIndices do
		    ns[k][i]:= Rmult(Rnegate( A[j,k] ),Rconvert(op(Flag,k)))
		end_for
            end_if
        end_for;
        t:= [];
	for k in charIndices do
	    t:= t . [MnewThis(c,1,ns[k])]
	end_for;
	A::_plus(s,op(t))
    end_if
end_proc:

# end of file #
